import 'package:clean_android_design/clean_android_design.dart';
import 'package:clean_android_design/src/dialog/datetime_picker_dialog.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';

import 'default_field_widget.dart';

class DatetimeFieldWidget extends StatefulWidget implements IListItem {
  final bool isFirst;
  final bool isLast;
  final Text title;
  final bool withDate;
  final bool withTime;

  final String emptyDateString;
  final String emptyDateStringEditable;

  final FieldValidation validation;
  final void Function(DateTime value) saveValue;
  final DateTime Function() getValue;

  final bool isEditable;
  final bool isDisabled;

  const DatetimeFieldWidget({
    Key key,
    this.title,
    this.saveValue,
    @required this.getValue,
    this.withDate = true,
    this.withTime = true,
    this.validation,
    @required this.isEditable,
    this.emptyDateString = "-",
    this.emptyDateStringEditable = "Datum",
    this.isDisabled = false,
  })  : assert(title != null),
        assert(getValue != null),
        assert(!(isEditable ?? true) || saveValue != null),
        isLast = null,
        isFirst = null,
        super(key: key);

  const DatetimeFieldWidget._copy({
    Key key,
    this.isFirst,
    this.isLast,
    this.title,
    this.withDate,
    this.withTime,
    this.saveValue,
    this.getValue,
    this.isEditable,
    this.validation,
    this.emptyDateString,
    this.emptyDateStringEditable,
    this.isDisabled,
  }) : super(key: key);

  @override
  _DatetimeFieldWidgetState createState() => _DatetimeFieldWidgetState();

  @override
  IListItem copyWithPosition({bool isFirst, bool isLast}) {
    return DatetimeFieldWidget._copy(
      key: this.key,
      title: this.title,
      getValue: this.getValue,
      saveValue: this.saveValue,
      withDate: this.withDate,
      withTime: this.withTime,
      isEditable: this.isEditable,
      validation: this.validation,
      emptyDateString: this.emptyDateString,
      emptyDateStringEditable: this.emptyDateStringEditable,
      isDisabled: this.isDisabled,
      isFirst: isFirst ?? this.isFirst,
      isLast: isLast ?? this.isLast,
    );
  }
}

class _DatetimeFieldWidgetState extends State<DatetimeFieldWidget> {
  String errorMessage;

  bool get isEditable => widget.isEditable && !widget.isDisabled;

  @override
  void initState() {
    super.initState();

    widget.validation?.connectToField(
      onValidate: (errorMessage) {
        if (mounted) setState(() => this.errorMessage = errorMessage);
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return DefaultFieldWidget(
      isFirst: widget.isFirst,
      isLast: widget.isLast,
      separationLine: true,
      title: widget.title,
      childPadding:
          isEditable ? EdgeInsets.only(left: 30, right: 30, bottom: 3) : null,
      child: Column(
        children: <Widget>[
          Row(
            children: <Widget>[
              _getDateWidget(context),
              SizedBox(
                width: widget.withDate ? 15 : 0,
              ),
              _getTimeWidget(),
            ],
          ),
          errorMessage != null
              ? Align(
                  alignment: Alignment.centerLeft,
                  child: Padding(
                    padding: const EdgeInsets.only(top: 6),
                    child: Text(
                      errorMessage,
                      style: TextStyles.textStyleSize(12).copyWith(
                          color: CleanAndroidDesignDefaultColors.error),
                    ),
                  ),
                )
              : Container(),
        ],
      ),
    );
  }

  Widget _getDateWidget(BuildContext context) {
    if (!widget.withDate) return Container();

    var date = widget.getValue();
    var dateString = date == null
        ? (isEditable ? widget.emptyDateStringEditable : widget.emptyDateString)
        : DateFormat("EEE dd.MM.yyy", "de_DE").format(date);
    return isEditable
        ? SimpleButtonWidget(
            borderColor: errorMessage != null
                ? CleanAndroidDesignDefaultColors.error
                : null,
            width: 145,
            alignment: MainAxisAlignment.start,
            child: Text(
              dateString,
              style: TextStyles.textStyleSize(16).copyWith(
                color: CleanAndroidDesignDefaultColors.black,
              ),
            ),
            onPressed: () {
              FocusScope.of(context).unfocus();
              showCustomDialog(
                  context,
                  DatetimePickerDialog(
                    datetime: date,
                    setDatetime: (date) =>
                        setState(() => widget.saveValue(date)),
                  ));
            },
          )
        : AdaptiveText(
            dateString,
            style: TextStyles.textStyleSize(17).copyWith(
              color: widget.isDisabled
                  ? Colors.grey[500]
                  : CleanAndroidDesignDefaultColors.black,
            ),
          );
  }

  Widget _getTimeWidget() {
    if (!widget.withTime) return Container();

    var date = widget.getValue();
    var dateString = date == null
        ? (isEditable ? "Uhrzeit" : (widget.withDate ? "" : "-"))
        : DateFormat("HH:mm").format(date);
    return isEditable
        ? SimpleButtonWidget(
            borderColor: errorMessage != null
                ? CleanAndroidDesignDefaultColors.error
                : null,
            width: date == null ? 90 : 70,
            alignment: MainAxisAlignment.start,
            child: Text(
              dateString,
              style: TextStyles.textStyleSize(16)
                  .copyWith(color: CleanAndroidDesignDefaultColors.black),
            ),
            onPressed: () {
              showCustomDialog(
                  context,
                  DatetimePickerDialog(
                    datetime: date,
                    timePicker: true,
                    setDatetime: (date) =>
                        setState(() => widget.saveValue(date)),
                  ));
            },
          )
        : AdaptiveText(
            dateString,
            style: TextStyles.textStyleSize(17).copyWith(
              color: widget.isDisabled
                  ? Colors.grey[500]
                  : CleanAndroidDesignDefaultColors.black,
            ),
          );
  }
}
